/*******************************************************************************
* File Name: URTINT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "URT.h"



/***************************************
* Custom Declarations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if (URT_RX_INTERRUPT_ENABLED && (URT_RX_ENABLED || URT_HD_ENABLED))
    /*******************************************************************************
    * Function Name: URT_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_rxBuffer - RAM buffer pointer for save received data.
    *  URT_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  URT_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  URT_rxBufferOverflow - software overflow flag. Set to one
    *     when URT_rxBufferWrite index overtakes
    *     URT_rxBufferRead index.
    *  URT_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when URT_rxBufferWrite is equal to
    *    URT_rxBufferRead
    *  URT_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  URT_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(URT_RXISR)
    {
        uint8 readData;
        uint8 readStatus;
        uint8 increment_pointer = 0u;

    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef URT_RXISR_ENTRY_CALLBACK
        URT_RXISR_EntryCallback();
    #endif /* URT_RXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START URT_RXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        do
        {
            /* Read receiver status register */
            readStatus = URT_RXSTATUS_REG;
            /* Copy the same status to readData variable for backward compatibility support 
            *  of the user code in URT_RXISR_ERROR` section. 
            */
            readData = readStatus;

            if((readStatus & (URT_RX_STS_BREAK | 
                            URT_RX_STS_PAR_ERROR |
                            URT_RX_STS_STOP_ERROR | 
                            URT_RX_STS_OVERRUN)) != 0u)
            {
                /* ERROR handling. */
                URT_errorStatus |= readStatus & ( URT_RX_STS_BREAK | 
                                                            URT_RX_STS_PAR_ERROR | 
                                                            URT_RX_STS_STOP_ERROR | 
                                                            URT_RX_STS_OVERRUN);
                /* `#START URT_RXISR_ERROR` */

                /* `#END` */
                
            #ifdef URT_RXISR_ERROR_CALLBACK
                URT_RXISR_ERROR_Callback();
            #endif /* URT_RXISR_ERROR_CALLBACK */
            }
            
            if((readStatus & URT_RX_STS_FIFO_NOTEMPTY) != 0u)
            {
                /* Read data from the RX data register */
                readData = URT_RXDATA_REG;
            #if (URT_RXHW_ADDRESS_ENABLED)
                if(URT_rxAddressMode == (uint8)URT__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readStatus & URT_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readStatus & URT_RX_STS_ADDR_MATCH) != 0u)
                        {
                            URT_rxAddressDetected = 1u;
                        }
                        else
                        {
                            URT_rxAddressDetected = 0u;
                        }
                    }
                    if(URT_rxAddressDetected != 0u)
                    {   /* Store only addressed data */
                        URT_rxBuffer[URT_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* Without software addressing */
                {
                    URT_rxBuffer[URT_rxBufferWrite] = readData;
                    increment_pointer = 1u;
                }
            #else  /* Without addressing */
                URT_rxBuffer[URT_rxBufferWrite] = readData;
                increment_pointer = 1u;
            #endif /* (URT_RXHW_ADDRESS_ENABLED) */

                /* Do not increment buffer pointer when skip not addressed data */
                if(increment_pointer != 0u)
                {
                    if(URT_rxBufferLoopDetect != 0u)
                    {   /* Set Software Buffer status Overflow */
                        URT_rxBufferOverflow = 1u;
                    }
                    /* Set next pointer. */
                    URT_rxBufferWrite++;

                    /* Check pointer for a loop condition */
                    if(URT_rxBufferWrite >= URT_RX_BUFFER_SIZE)
                    {
                        URT_rxBufferWrite = 0u;
                    }

                    /* Detect pre-overload condition and set flag */
                    if(URT_rxBufferWrite == URT_rxBufferRead)
                    {
                        URT_rxBufferLoopDetect = 1u;
                        /* When Hardware Flow Control selected */
                        #if (URT_FLOW_CONTROL != 0u)
                            /* Disable RX interrupt mask, it is enabled when user read data from the buffer using APIs */
                            URT_RXSTATUS_MASK_REG  &= (uint8)~URT_RX_STS_FIFO_NOTEMPTY;
                            CyIntClearPending(URT_RX_VECT_NUM);
                            break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                        #endif /* (URT_FLOW_CONTROL != 0u) */
                    }
                }
            }
        }while((readStatus & URT_RX_STS_FIFO_NOTEMPTY) != 0u);

        /* User code required at end of ISR (Optional) */
        /* `#START URT_RXISR_END` */

        /* `#END` */

    #ifdef URT_RXISR_EXIT_CALLBACK
        URT_RXISR_ExitCallback();
    #endif /* URT_RXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
    }
    
#endif /* (URT_RX_INTERRUPT_ENABLED && (URT_RX_ENABLED || URT_HD_ENABLED)) */


#if (URT_TX_INTERRUPT_ENABLED && URT_TX_ENABLED)
    /*******************************************************************************
    * Function Name: URT_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_txBuffer - RAM buffer pointer for transmit data from.
    *  URT_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmitted byte.
    *  URT_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(URT_TXISR)
    {
    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef URT_TXISR_ENTRY_CALLBACK
        URT_TXISR_EntryCallback();
    #endif /* URT_TXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START URT_TXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        while((URT_txBufferRead != URT_txBufferWrite) &&
             ((URT_TXSTATUS_REG & URT_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer wrap around */
            if(URT_txBufferRead >= URT_TX_BUFFER_SIZE)
            {
                URT_txBufferRead = 0u;
            }

            URT_TXDATA_REG = URT_txBuffer[URT_txBufferRead];

            /* Set next pointer */
            URT_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START URT_TXISR_END` */

        /* `#END` */

    #ifdef URT_TXISR_EXIT_CALLBACK
        URT_TXISR_ExitCallback();
    #endif /* URT_TXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
   }
#endif /* (URT_TX_INTERRUPT_ENABLED && URT_TX_ENABLED) */


/* [] END OF FILE */
