/*******************************************************************************
* File Name: URT_PM.c
* Version 2.50
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "URT.h"


/***************************************
* Local data allocation
***************************************/

static URT_BACKUP_STRUCT  URT_backup =
{
    /* enableState - disabled */
    0u,
};



/*******************************************************************************
* Function Name: URT_SaveConfig
********************************************************************************
*
* Summary:
*  This function saves the component nonretention control register.
*  Does not save the FIFO which is a set of nonretention registers.
*  This function is called by the URT_Sleep() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  URT_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void URT_SaveConfig(void)
{
    #if(URT_CONTROL_REG_REMOVED == 0u)
        URT_backup.cr = URT_CONTROL_REG;
    #endif /* End URT_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: URT_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the nonretention control register except FIFO.
*  Does not restore the FIFO which is a set of nonretention registers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  URT_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
* Notes:
*  If this function is called without calling URT_SaveConfig() 
*  first, the data loaded may be incorrect.
*
*******************************************************************************/
void URT_RestoreConfig(void)
{
    #if(URT_CONTROL_REG_REMOVED == 0u)
        URT_CONTROL_REG = URT_backup.cr;
    #endif /* End URT_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: URT_Sleep
********************************************************************************
*
* Summary:
*  This is the preferred API to prepare the component for sleep. 
*  The URT_Sleep() API saves the current component state. Then it
*  calls the URT_Stop() function and calls 
*  URT_SaveConfig() to save the hardware configuration.
*  Call the URT_Sleep() function before calling the CyPmSleep() 
*  or the CyPmHibernate() function. 
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  URT_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void URT_Sleep(void)
{
    #if(URT_RX_ENABLED || URT_HD_ENABLED)
        if((URT_RXSTATUS_ACTL_REG  & URT_INT_ENABLE) != 0u)
        {
            URT_backup.enableState = 1u;
        }
        else
        {
            URT_backup.enableState = 0u;
        }
    #else
        if((URT_TXSTATUS_ACTL_REG  & URT_INT_ENABLE) !=0u)
        {
            URT_backup.enableState = 1u;
        }
        else
        {
            URT_backup.enableState = 0u;
        }
    #endif /* End URT_RX_ENABLED || URT_HD_ENABLED*/

    URT_Stop();
    URT_SaveConfig();
}


/*******************************************************************************
* Function Name: URT_Wakeup
********************************************************************************
*
* Summary:
*  This is the preferred API to restore the component to the state when 
*  URT_Sleep() was called. The URT_Wakeup() function
*  calls the URT_RestoreConfig() function to restore the 
*  configuration. If the component was enabled before the 
*  URT_Sleep() function was called, the URT_Wakeup()
*  function will also re-enable the component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  URT_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void URT_Wakeup(void)
{
    URT_RestoreConfig();
    #if( (URT_RX_ENABLED) || (URT_HD_ENABLED) )
        URT_ClearRxBuffer();
    #endif /* End (URT_RX_ENABLED) || (URT_HD_ENABLED) */
    #if(URT_TX_ENABLED || URT_HD_ENABLED)
        URT_ClearTxBuffer();
    #endif /* End URT_TX_ENABLED || URT_HD_ENABLED */

    if(URT_backup.enableState != 0u)
    {
        URT_Enable();
    }
}


/* [] END OF FILE */
