/*******************************************************************************
* File Name: URT.c
* Version 2.50
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "URT.h"
#if (URT_INTERNAL_CLOCK_USED)
    #include "URT_IntClock.h"
#endif /* End URT_INTERNAL_CLOCK_USED */


/***************************************
* Global data allocation
***************************************/

uint8 URT_initVar = 0u;

#if (URT_TX_INTERRUPT_ENABLED && URT_TX_ENABLED)
    volatile uint8 URT_txBuffer[URT_TX_BUFFER_SIZE];
    volatile uint8 URT_txBufferRead = 0u;
    uint8 URT_txBufferWrite = 0u;
#endif /* (URT_TX_INTERRUPT_ENABLED && URT_TX_ENABLED) */

#if (URT_RX_INTERRUPT_ENABLED && (URT_RX_ENABLED || URT_HD_ENABLED))
    uint8 URT_errorStatus = 0u;
    volatile uint8 URT_rxBuffer[URT_RX_BUFFER_SIZE];
    volatile uint8 URT_rxBufferRead  = 0u;
    volatile uint8 URT_rxBufferWrite = 0u;
    volatile uint8 URT_rxBufferLoopDetect = 0u;
    volatile uint8 URT_rxBufferOverflow   = 0u;
    #if (URT_RXHW_ADDRESS_ENABLED)
        volatile uint8 URT_rxAddressMode = URT_RX_ADDRESS_MODE;
        volatile uint8 URT_rxAddressDetected = 0u;
    #endif /* (URT_RXHW_ADDRESS_ENABLED) */
#endif /* (URT_RX_INTERRUPT_ENABLED && (URT_RX_ENABLED || URT_HD_ENABLED)) */


/*******************************************************************************
* Function Name: URT_Start
********************************************************************************
*
* Summary:
*  This is the preferred method to begin component operation.
*  URT_Start() sets the initVar variable, calls the
*  URT_Init() function, and then calls the
*  URT_Enable() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The URT_intiVar variable is used to indicate initial
*  configuration of this component. The variable is initialized to zero (0u)
*  and set to one (1u) the first time URT_Start() is called. This
*  allows for component initialization without re-initialization in all
*  subsequent calls to the URT_Start() routine.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void URT_Start(void) 
{
    /* If not initialized then initialize all required hardware and software */
    if(URT_initVar == 0u)
    {
        URT_Init();
        URT_initVar = 1u;
    }

    URT_Enable();
}


/*******************************************************************************
* Function Name: URT_Init
********************************************************************************
*
* Summary:
*  Initializes or restores the component according to the customizer Configure
*  dialog settings. It is not necessary to call URT_Init() because
*  the URT_Start() API calls this function and is the preferred
*  method to begin component operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void URT_Init(void) 
{
    #if(URT_RX_ENABLED || URT_HD_ENABLED)

        #if (URT_RX_INTERRUPT_ENABLED)
            /* Set RX interrupt vector and priority */
            (void) CyIntSetVector(URT_RX_VECT_NUM, &URT_RXISR);
            CyIntSetPriority(URT_RX_VECT_NUM, URT_RX_PRIOR_NUM);
            URT_errorStatus = 0u;
        #endif /* (URT_RX_INTERRUPT_ENABLED) */

        #if (URT_RXHW_ADDRESS_ENABLED)
            URT_SetRxAddressMode(URT_RX_ADDRESS_MODE);
            URT_SetRxAddress1(URT_RX_HW_ADDRESS1);
            URT_SetRxAddress2(URT_RX_HW_ADDRESS2);
        #endif /* End URT_RXHW_ADDRESS_ENABLED */

        /* Init Count7 period */
        URT_RXBITCTR_PERIOD_REG = URT_RXBITCTR_INIT;
        /* Configure the Initial RX interrupt mask */
        URT_RXSTATUS_MASK_REG  = URT_INIT_RX_INTERRUPTS_MASK;
    #endif /* End URT_RX_ENABLED || URT_HD_ENABLED*/

    #if(URT_TX_ENABLED)
        #if (URT_TX_INTERRUPT_ENABLED)
            /* Set TX interrupt vector and priority */
            (void) CyIntSetVector(URT_TX_VECT_NUM, &URT_TXISR);
            CyIntSetPriority(URT_TX_VECT_NUM, URT_TX_PRIOR_NUM);
        #endif /* (URT_TX_INTERRUPT_ENABLED) */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if (URT_TXCLKGEN_DP)
            URT_TXBITCLKGEN_CTR_REG = URT_BIT_CENTER;
            URT_TXBITCLKTX_COMPLETE_REG = ((URT_NUMBER_OF_DATA_BITS +
                        URT_NUMBER_OF_START_BIT) * URT_OVER_SAMPLE_COUNT) - 1u;
        #else
            URT_TXBITCTR_PERIOD_REG = ((URT_NUMBER_OF_DATA_BITS +
                        URT_NUMBER_OF_START_BIT) * URT_OVER_SAMPLE_8) - 1u;
        #endif /* End URT_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if (URT_TX_INTERRUPT_ENABLED)
            URT_TXSTATUS_MASK_REG = URT_TX_STS_FIFO_EMPTY;
        #else
            URT_TXSTATUS_MASK_REG = URT_INIT_TX_INTERRUPTS_MASK;
        #endif /*End URT_TX_INTERRUPT_ENABLED*/

    #endif /* End URT_TX_ENABLED */

    #if(URT_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        URT_WriteControlRegister( \
            (URT_ReadControlRegister() & (uint8)~URT_CTRL_PARITY_TYPE_MASK) | \
            (uint8)(URT_PARITY_TYPE << URT_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End URT_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: URT_Enable
********************************************************************************
*
* Summary:
*  Activates the hardware and begins component operation. It is not necessary
*  to call URT_Enable() because the URT_Start() API
*  calls this function, which is the preferred method to begin component
*  operation.

* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  URT_rxAddressDetected - set to initial state (0).
*
*******************************************************************************/
void URT_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    #if (URT_RX_ENABLED || URT_HD_ENABLED)
        /* RX Counter (Count7) Enable */
        URT_RXBITCTR_CONTROL_REG |= URT_CNTR_ENABLE;

        /* Enable the RX Interrupt */
        URT_RXSTATUS_ACTL_REG  |= URT_INT_ENABLE;

        #if (URT_RX_INTERRUPT_ENABLED)
            URT_EnableRxInt();

            #if (URT_RXHW_ADDRESS_ENABLED)
                URT_rxAddressDetected = 0u;
            #endif /* (URT_RXHW_ADDRESS_ENABLED) */
        #endif /* (URT_RX_INTERRUPT_ENABLED) */
    #endif /* (URT_RX_ENABLED || URT_HD_ENABLED) */

    #if(URT_TX_ENABLED)
        /* TX Counter (DP/Count7) Enable */
        #if(!URT_TXCLKGEN_DP)
            URT_TXBITCTR_CONTROL_REG |= URT_CNTR_ENABLE;
        #endif /* End URT_TXCLKGEN_DP */

        /* Enable the TX Interrupt */
        URT_TXSTATUS_ACTL_REG |= URT_INT_ENABLE;
        #if (URT_TX_INTERRUPT_ENABLED)
            URT_ClearPendingTxInt(); /* Clear history of TX_NOT_EMPTY */
            URT_EnableTxInt();
        #endif /* (URT_TX_INTERRUPT_ENABLED) */
     #endif /* (URT_TX_INTERRUPT_ENABLED) */

    #if (URT_INTERNAL_CLOCK_USED)
        URT_IntClock_Start();  /* Enable the clock */
    #endif /* (URT_INTERNAL_CLOCK_USED) */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: URT_Stop
********************************************************************************
*
* Summary:
*  Disables the UART operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void URT_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /* Write Bit Counter Disable */
    #if (URT_RX_ENABLED || URT_HD_ENABLED)
        URT_RXBITCTR_CONTROL_REG &= (uint8) ~URT_CNTR_ENABLE;
    #endif /* (URT_RX_ENABLED || URT_HD_ENABLED) */

    #if (URT_TX_ENABLED)
        #if(!URT_TXCLKGEN_DP)
            URT_TXBITCTR_CONTROL_REG &= (uint8) ~URT_CNTR_ENABLE;
        #endif /* (!URT_TXCLKGEN_DP) */
    #endif /* (URT_TX_ENABLED) */

    #if (URT_INTERNAL_CLOCK_USED)
        URT_IntClock_Stop();   /* Disable the clock */
    #endif /* (URT_INTERNAL_CLOCK_USED) */

    /* Disable internal interrupt component */
    #if (URT_RX_ENABLED || URT_HD_ENABLED)
        URT_RXSTATUS_ACTL_REG  &= (uint8) ~URT_INT_ENABLE;

        #if (URT_RX_INTERRUPT_ENABLED)
            URT_DisableRxInt();
        #endif /* (URT_RX_INTERRUPT_ENABLED) */
    #endif /* (URT_RX_ENABLED || URT_HD_ENABLED) */

    #if (URT_TX_ENABLED)
        URT_TXSTATUS_ACTL_REG &= (uint8) ~URT_INT_ENABLE;

        #if (URT_TX_INTERRUPT_ENABLED)
            URT_DisableTxInt();
        #endif /* (URT_TX_INTERRUPT_ENABLED) */
    #endif /* (URT_TX_ENABLED) */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: URT_ReadControlRegister
********************************************************************************
*
* Summary:
*  Returns the current value of the control register.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the control register.
*
*******************************************************************************/
uint8 URT_ReadControlRegister(void) 
{
    #if (URT_CONTROL_REG_REMOVED)
        return(0u);
    #else
        return(URT_CONTROL_REG);
    #endif /* (URT_CONTROL_REG_REMOVED) */
}


/*******************************************************************************
* Function Name: URT_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  URT_WriteControlRegister(uint8 control) 
{
    #if (URT_CONTROL_REG_REMOVED)
        if(0u != control)
        {
            /* Suppress compiler warning */
        }
    #else
       URT_CONTROL_REG = control;
    #endif /* (URT_CONTROL_REG_REMOVED) */
}


#if(URT_RX_ENABLED || URT_HD_ENABLED)
    /*******************************************************************************
    * Function Name: URT_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configures the RX interrupt sources enabled.
    *
    * Parameters:
    *  IntSrc:  Bit field containing the RX interrupts to enable. Based on the 
    *  bit-field arrangement of the status register. This value must be a 
    *  combination of status register bit-masks shown below:
    *      URT_RX_STS_FIFO_NOTEMPTY    Interrupt on byte received.
    *      URT_RX_STS_PAR_ERROR        Interrupt on parity error.
    *      URT_RX_STS_STOP_ERROR       Interrupt on stop error.
    *      URT_RX_STS_BREAK            Interrupt on break.
    *      URT_RX_STS_OVERRUN          Interrupt on overrun error.
    *      URT_RX_STS_ADDR_MATCH       Interrupt on address match.
    *      URT_RX_STS_MRKSPC           Interrupt on address detect.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void URT_SetRxInterruptMode(uint8 intSrc) 
    {
        URT_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: URT_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns the next byte of received data. This function returns data without
    *  checking the status. You must check the status separately.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  URT_rxBuffer - RAM buffer pointer for save received data.
    *  URT_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  URT_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  URT_rxBufferLoopDetect - cleared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 URT_ReadRxData(void) 
    {
        uint8 rxData;

    #if (URT_RX_INTERRUPT_ENABLED)

        uint8 locRxBufferRead;
        uint8 locRxBufferWrite;

        /* Protect variables that could change on interrupt */
        URT_DisableRxInt();

        locRxBufferRead  = URT_rxBufferRead;
        locRxBufferWrite = URT_rxBufferWrite;

        if( (URT_rxBufferLoopDetect != 0u) || (locRxBufferRead != locRxBufferWrite) )
        {
            rxData = URT_rxBuffer[locRxBufferRead];
            locRxBufferRead++;

            if(locRxBufferRead >= URT_RX_BUFFER_SIZE)
            {
                locRxBufferRead = 0u;
            }
            /* Update the real pointer */
            URT_rxBufferRead = locRxBufferRead;

            if(URT_rxBufferLoopDetect != 0u)
            {
                URT_rxBufferLoopDetect = 0u;
                #if ((URT_RX_INTERRUPT_ENABLED) && (URT_FLOW_CONTROL != 0u))
                    /* When Hardware Flow Control selected - return RX mask */
                    #if( URT_HD_ENABLED )
                        if((URT_CONTROL_REG & URT_CTRL_HD_SEND) == 0u)
                        {   /* In Half duplex mode return RX mask only in RX
                            *  configuration set, otherwise
                            *  mask will be returned in LoadRxConfig() API.
                            */
                            URT_RXSTATUS_MASK_REG  |= URT_RX_STS_FIFO_NOTEMPTY;
                        }
                    #else
                        URT_RXSTATUS_MASK_REG  |= URT_RX_STS_FIFO_NOTEMPTY;
                    #endif /* end URT_HD_ENABLED */
                #endif /* ((URT_RX_INTERRUPT_ENABLED) && (URT_FLOW_CONTROL != 0u)) */
            }
        }
        else
        {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit */
            rxData = URT_RXDATA_REG;
        }

        URT_EnableRxInt();

    #else

        /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit */
        rxData = URT_RXDATA_REG;

    #endif /* (URT_RX_INTERRUPT_ENABLED) */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: URT_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Returns the current state of the receiver status register and the software
    *  buffer overflow status.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Side Effect:
    *  All status register bits are clear-on-read except
    *  URT_RX_STS_FIFO_NOTEMPTY.
    *  URT_RX_STS_FIFO_NOTEMPTY clears immediately after RX data
    *  register read.
    *
    * Global Variables:
    *  URT_rxBufferOverflow - used to indicate overload condition.
    *   It set to one in RX interrupt when there isn't free space in
    *   URT_rxBufferRead to write new data. This condition returned
    *   and cleared to zero by this API as an
    *   URT_RX_STS_SOFT_BUFF_OVER bit along with RX Status register
    *   bits.
    *
    *******************************************************************************/
    uint8 URT_ReadRxStatus(void) 
    {
        uint8 status;

        status = URT_RXSTATUS_REG & URT_RX_HW_MASK;

    #if (URT_RX_INTERRUPT_ENABLED)
        if(URT_rxBufferOverflow != 0u)
        {
            status |= URT_RX_STS_SOFT_BUFF_OVER;
            URT_rxBufferOverflow = 0u;
        }
    #endif /* (URT_RX_INTERRUPT_ENABLED) */

        return(status);
    }


    /*******************************************************************************
    * Function Name: URT_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Returns the last received byte of data. URT_GetChar() is
    *  designed for ASCII characters and returns a uint8 where 1 to 255 are values
    *  for valid characters and 0 indicates an error occurred or no data is present.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  URT_rxBuffer - RAM buffer pointer for save received data.
    *  URT_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  URT_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  URT_rxBufferLoopDetect - cleared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 URT_GetChar(void) 
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

    #if (URT_RX_INTERRUPT_ENABLED)
        uint8 locRxBufferRead;
        uint8 locRxBufferWrite;

        /* Protect variables that could change on interrupt */
        URT_DisableRxInt();

        locRxBufferRead  = URT_rxBufferRead;
        locRxBufferWrite = URT_rxBufferWrite;

        if( (URT_rxBufferLoopDetect != 0u) || (locRxBufferRead != locRxBufferWrite) )
        {
            rxData = URT_rxBuffer[locRxBufferRead];
            locRxBufferRead++;
            if(locRxBufferRead >= URT_RX_BUFFER_SIZE)
            {
                locRxBufferRead = 0u;
            }
            /* Update the real pointer */
            URT_rxBufferRead = locRxBufferRead;

            if(URT_rxBufferLoopDetect != 0u)
            {
                URT_rxBufferLoopDetect = 0u;
                #if( (URT_RX_INTERRUPT_ENABLED) && (URT_FLOW_CONTROL != 0u) )
                    /* When Hardware Flow Control selected - return RX mask */
                    #if( URT_HD_ENABLED )
                        if((URT_CONTROL_REG & URT_CTRL_HD_SEND) == 0u)
                        {   /* In Half duplex mode return RX mask only if
                            *  RX configuration set, otherwise
                            *  mask will be returned in LoadRxConfig() API.
                            */
                            URT_RXSTATUS_MASK_REG |= URT_RX_STS_FIFO_NOTEMPTY;
                        }
                    #else
                        URT_RXSTATUS_MASK_REG |= URT_RX_STS_FIFO_NOTEMPTY;
                    #endif /* end URT_HD_ENABLED */
                #endif /* URT_RX_INTERRUPT_ENABLED and Hardware flow control*/
            }

        }
        else
        {   rxStatus = URT_RXSTATUS_REG;
            if((rxStatus & URT_RX_STS_FIFO_NOTEMPTY) != 0u)
            {   /* Read received data from FIFO */
                rxData = URT_RXDATA_REG;
                /*Check status on error*/
                if((rxStatus & (URT_RX_STS_BREAK | URT_RX_STS_PAR_ERROR |
                                URT_RX_STS_STOP_ERROR | URT_RX_STS_OVERRUN)) != 0u)
                {
                    rxData = 0u;
                }
            }
        }

        URT_EnableRxInt();

    #else

        rxStatus =URT_RXSTATUS_REG;
        if((rxStatus & URT_RX_STS_FIFO_NOTEMPTY) != 0u)
        {
            /* Read received data from FIFO */
            rxData = URT_RXDATA_REG;

            /*Check status on error*/
            if((rxStatus & (URT_RX_STS_BREAK | URT_RX_STS_PAR_ERROR |
                            URT_RX_STS_STOP_ERROR | URT_RX_STS_OVERRUN)) != 0u)
            {
                rxData = 0u;
            }
        }
    #endif /* (URT_RX_INTERRUPT_ENABLED) */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: URT_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, returns received character and error
    *  condition.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains status and LSB contains UART RX data. If the MSB is nonzero,
    *  an error has occurred.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 URT_GetByte(void) 
    {
        
    #if (URT_RX_INTERRUPT_ENABLED)
        uint16 locErrorStatus;
        /* Protect variables that could change on interrupt */
        URT_DisableRxInt();
        locErrorStatus = (uint16)URT_errorStatus;
        URT_errorStatus = 0u;
        URT_EnableRxInt();
        return ( (uint16)(locErrorStatus << 8u) | URT_ReadRxData() );
    #else
        return ( ((uint16)URT_ReadRxStatus() << 8u) | URT_ReadRxData() );
    #endif /* URT_RX_INTERRUPT_ENABLED */
        
    }


    /*******************************************************************************
    * Function Name: URT_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of received bytes available in the RX buffer.
    *  * RX software buffer is disabled (RX Buffer Size parameter is equal to 4): 
    *    returns 0 for empty RX FIFO or 1 for not empty RX FIFO.
    *  * RX software buffer is enabled: returns the number of bytes available in 
    *    the RX software buffer. Bytes available in the RX FIFO do not take to 
    *    account.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Number of bytes in the RX buffer. 
    *    Return value type depends on RX Buffer Size parameter.
    *
    * Global Variables:
    *  URT_rxBufferWrite - used to calculate left bytes.
    *  URT_rxBufferRead - used to calculate left bytes.
    *  URT_rxBufferLoopDetect - checked to decide left bytes amount.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 URT_GetRxBufferSize(void)
                                                            
    {
        uint8 size;

    #if (URT_RX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt */
        URT_DisableRxInt();

        if(URT_rxBufferRead == URT_rxBufferWrite)
        {
            if(URT_rxBufferLoopDetect != 0u)
            {
                size = URT_RX_BUFFER_SIZE;
            }
            else
            {
                size = 0u;
            }
        }
        else if(URT_rxBufferRead < URT_rxBufferWrite)
        {
            size = (URT_rxBufferWrite - URT_rxBufferRead);
        }
        else
        {
            size = (URT_RX_BUFFER_SIZE - URT_rxBufferRead) + URT_rxBufferWrite;
        }

        URT_EnableRxInt();

    #else

        /* We can only know if there is data in the fifo. */
        size = ((URT_RXSTATUS_REG & URT_RX_STS_FIFO_NOTEMPTY) != 0u) ? 1u : 0u;

    #endif /* (URT_RX_INTERRUPT_ENABLED) */

        return(size);
    }


    /*******************************************************************************
    * Function Name: URT_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the receiver memory buffer and hardware RX FIFO of all received data.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_rxBufferWrite - cleared to zero.
    *  URT_rxBufferRead - cleared to zero.
    *  URT_rxBufferLoopDetect - cleared to zero.
    *  URT_rxBufferOverflow - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM or FIFO buffer will be lost.
    *
    *******************************************************************************/
    void URT_ClearRxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* Clear the HW FIFO */
        enableInterrupts = CyEnterCriticalSection();
        URT_RXDATA_AUX_CTL_REG |= (uint8)  URT_RX_FIFO_CLR;
        URT_RXDATA_AUX_CTL_REG &= (uint8) ~URT_RX_FIFO_CLR;
        CyExitCriticalSection(enableInterrupts);

    #if (URT_RX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt. */
        URT_DisableRxInt();

        URT_rxBufferRead = 0u;
        URT_rxBufferWrite = 0u;
        URT_rxBufferLoopDetect = 0u;
        URT_rxBufferOverflow = 0u;

        URT_EnableRxInt();

    #endif /* (URT_RX_INTERRUPT_ENABLED) */

    }


    /*******************************************************************************
    * Function Name: URT_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Sets the software controlled Addressing mode used by the RX portion of the
    *  UART.
    *
    * Parameters:
    *  addressMode: Enumerated value indicating the mode of RX addressing
    *  URT__B_UART__AM_SW_BYTE_BYTE -  Software Byte-by-Byte address
    *                                               detection
    *  URT__B_UART__AM_SW_DETECT_TO_BUFFER - Software Detect to Buffer
    *                                               address detection
    *  URT__B_UART__AM_HW_BYTE_BY_BYTE - Hardware Byte-by-Byte address
    *                                               detection
    *  URT__B_UART__AM_HW_DETECT_TO_BUFFER - Hardware Detect to Buffer
    *                                               address detection
    *  URT__B_UART__AM_NONE - No address detection
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_rxAddressMode - the parameter stored in this variable for
    *   the farther usage in RX ISR.
    *  URT_rxAddressDetected - set to initial state (0).
    *
    *******************************************************************************/
    void URT_SetRxAddressMode(uint8 addressMode)
                                                        
    {
        #if(URT_RXHW_ADDRESS_ENABLED)
            #if(URT_CONTROL_REG_REMOVED)
                if(0u != addressMode)
                {
                    /* Suppress compiler warning */
                }
            #else /* URT_CONTROL_REG_REMOVED */
                uint8 tmpCtrl;
                tmpCtrl = URT_CONTROL_REG & (uint8)~URT_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= (uint8)(addressMode << URT_CTRL_RXADDR_MODE0_SHIFT);
                URT_CONTROL_REG = tmpCtrl;

                #if(URT_RX_INTERRUPT_ENABLED && \
                   (URT_RXBUFFERSIZE > URT_FIFO_LENGTH) )
                    URT_rxAddressMode = addressMode;
                    URT_rxAddressDetected = 0u;
                #endif /* End URT_RXBUFFERSIZE > URT_FIFO_LENGTH*/
            #endif /* End URT_CONTROL_REG_REMOVED */
        #else /* URT_RXHW_ADDRESS_ENABLED */
            if(0u != addressMode)
            {
                /* Suppress compiler warning */
            }
        #endif /* End URT_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: URT_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Sets the first of two hardware-detectable receiver addresses.
    *
    * Parameters:
    *  address: Address #1 for hardware address detection.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void URT_SetRxAddress1(uint8 address) 
    {
        URT_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: URT_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Sets the second of two hardware-detectable receiver addresses.
    *
    * Parameters:
    *  address: Address #2 for hardware address detection.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void URT_SetRxAddress2(uint8 address) 
    {
        URT_RXADDRESS2_REG = address;
    }

#endif  /* URT_RX_ENABLED || URT_HD_ENABLED*/


#if( (URT_TX_ENABLED) || (URT_HD_ENABLED) )
    /*******************************************************************************
    * Function Name: URT_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configures the TX interrupt sources to be enabled, but does not enable the
    *  interrupt.
    *
    * Parameters:
    *  intSrc: Bit field containing the TX interrupt sources to enable
    *   URT_TX_STS_COMPLETE        Interrupt on TX byte complete
    *   URT_TX_STS_FIFO_EMPTY      Interrupt when TX FIFO is empty
    *   URT_TX_STS_FIFO_FULL       Interrupt when TX FIFO is full
    *   URT_TX_STS_FIFO_NOT_FULL   Interrupt when TX FIFO is not full
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void URT_SetTxInterruptMode(uint8 intSrc) 
    {
        URT_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: URT_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Places a byte of data into the transmit buffer to be sent when the bus is
    *  available without checking the TX status register. You must check status
    *  separately.
    *
    * Parameters:
    *  txDataByte: data byte
    *
    * Return:
    * None.
    *
    * Global Variables:
    *  URT_txBuffer - RAM buffer pointer for save data for transmission
    *  URT_txBufferWrite - cyclic index for write to txBuffer,
    *    incremented after each byte saved to buffer.
    *  URT_txBufferRead - cyclic index for read from txBuffer,
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  URT_initVar - checked to identify that the component has been
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void URT_WriteTxData(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(URT_initVar != 0u)
        {
        #if (URT_TX_INTERRUPT_ENABLED)

            /* Protect variables that could change on interrupt. */
            URT_DisableTxInt();

            if( (URT_txBufferRead == URT_txBufferWrite) &&
                ((URT_TXSTATUS_REG & URT_TX_STS_FIFO_FULL) == 0u) )
            {
                /* Add directly to the FIFO. */
                URT_TXDATA_REG = txDataByte;
            }
            else
            {
                if(URT_txBufferWrite >= URT_TX_BUFFER_SIZE)
                {
                    URT_txBufferWrite = 0u;
                }

                URT_txBuffer[URT_txBufferWrite] = txDataByte;

                /* Add to the software buffer. */
                URT_txBufferWrite++;
            }

            URT_EnableTxInt();

        #else

            /* Add directly to the FIFO. */
            URT_TXDATA_REG = txDataByte;

        #endif /*(URT_TX_INTERRUPT_ENABLED) */
        }
    }


    /*******************************************************************************
    * Function Name: URT_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Reads the status register for the TX portion of the UART.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the TX status register, which is cleared on read.
    *  It is up to the user to handle all bits in this return value accordingly,
    *  even if the bit was not enabled as an interrupt source the event happened
    *  and must be handled accordingly.
    *
    *******************************************************************************/
    uint8 URT_ReadTxStatus(void) 
    {
        return(URT_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: URT_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Puts a byte of data into the transmit buffer to be sent when the bus is
    *  available. This is a blocking API that waits until the TX buffer has room to
    *  hold the data.
    *
    * Parameters:
    *  txDataByte: Byte containing the data to transmit
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_txBuffer - RAM buffer pointer for save data for transmission
    *  URT_txBufferWrite - cyclic index for write to txBuffer,
    *     checked to identify free space in txBuffer and incremented after each byte
    *     saved to buffer.
    *  URT_txBufferRead - cyclic index for read from txBuffer,
    *     checked to identify free space in txBuffer.
    *  URT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void URT_PutChar(uint8 txDataByte) 
    {
    #if (URT_TX_INTERRUPT_ENABLED)
        /* The temporary output pointer is used since it takes two instructions
        *  to increment with a wrap, and we can't risk doing that with the real
        *  pointer and getting an interrupt in between instructions.
        */
        uint8 locTxBufferWrite;
        uint8 locTxBufferRead;

        do
        { /* Block if software buffer is full, so we don't overwrite. */

        #if ((URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3))
            /* Disable TX interrupt to protect variables from modification */
            URT_DisableTxInt();
        #endif /* (URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3) */

            locTxBufferWrite = URT_txBufferWrite;
            locTxBufferRead  = URT_txBufferRead;

        #if ((URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3))
            /* Enable interrupt to continue transmission */
            URT_EnableTxInt();
        #endif /* (URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3) */
        }
        while( (locTxBufferWrite < locTxBufferRead) ? (locTxBufferWrite == (locTxBufferRead - 1u)) :
                                ((locTxBufferWrite - locTxBufferRead) ==
                                (uint8)(URT_TX_BUFFER_SIZE - 1u)) );

        if( (locTxBufferRead == locTxBufferWrite) &&
            ((URT_TXSTATUS_REG & URT_TX_STS_FIFO_FULL) == 0u) )
        {
            /* Add directly to the FIFO */
            URT_TXDATA_REG = txDataByte;
        }
        else
        {
            if(locTxBufferWrite >= URT_TX_BUFFER_SIZE)
            {
                locTxBufferWrite = 0u;
            }
            /* Add to the software buffer. */
            URT_txBuffer[locTxBufferWrite] = txDataByte;
            locTxBufferWrite++;

            /* Finally, update the real output pointer */
        #if ((URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3))
            URT_DisableTxInt();
        #endif /* (URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3) */

            URT_txBufferWrite = locTxBufferWrite;

        #if ((URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3))
            URT_EnableTxInt();
        #endif /* (URT_TX_BUFFER_SIZE > URT_MAX_BYTE_VALUE) && (CY_PSOC3) */

            if(0u != (URT_TXSTATUS_REG & URT_TX_STS_FIFO_EMPTY))
            {
                /* Trigger TX interrupt to send software buffer */
                URT_SetPendingTxInt();
            }
        }

    #else

        while((URT_TXSTATUS_REG & URT_TX_STS_FIFO_FULL) != 0u)
        {
            /* Wait for room in the FIFO */
        }

        /* Add directly to the FIFO */
        URT_TXDATA_REG = txDataByte;

    #endif /* URT_TX_INTERRUPT_ENABLED */
    }


    /*******************************************************************************
    * Function Name: URT_PutString
    ********************************************************************************
    *
    * Summary:
    *  Sends a NULL terminated string to the TX buffer for transmission.
    *
    * Parameters:
    *  string[]: Pointer to the null terminated string array residing in RAM or ROM
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  If there is not enough memory in the TX buffer for the entire string, this
    *  function blocks until the last character of the string is loaded into the
    *  TX buffer.
    *
    *******************************************************************************/
    void URT_PutString(const char8 string[]) 
    {
        uint16 bufIndex = 0u;

        /* If not Initialized then skip this function */
        if(URT_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent */
            while(string[bufIndex] != (char8) 0)
            {
                URT_PutChar((uint8)string[bufIndex]);
                bufIndex++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: URT_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Places N bytes of data from a memory array into the TX buffer for
    *  transmission.
    *
    * Parameters:
    *  string[]: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of bytes to be transmitted. The type depends on TX Buffer
    *             Size parameter.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  If there is not enough memory in the TX buffer for the entire string, this
    *  function blocks until the last character of the string is loaded into the
    *  TX buffer.
    *
    *******************************************************************************/
    void URT_PutArray(const uint8 string[], uint8 byteCount)
                                                                    
    {
        uint8 bufIndex = 0u;

        /* If not Initialized then skip this function */
        if(URT_initVar != 0u)
        {
            while(bufIndex < byteCount)
            {
                URT_PutChar(string[bufIndex]);
                bufIndex++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: URT_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Writes a byte of data followed by a carriage return (0x0D) and line feed
    *  (0x0A) to the transmit buffer.
    *
    * Parameters:
    *  txDataByte: Data byte to transmit before the carriage return and line feed.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void URT_PutCRLF(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function */
        if(URT_initVar != 0u)
        {
            URT_PutChar(txDataByte);
            URT_PutChar(0x0Du);
            URT_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: URT_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of bytes in the TX buffer which are waiting to be 
    *  transmitted.
    *  * TX software buffer is disabled (TX Buffer Size parameter is equal to 4): 
    *    returns 0 for empty TX FIFO, 1 for not full TX FIFO or 4 for full TX FIFO.
    *  * TX software buffer is enabled: returns the number of bytes in the TX 
    *    software buffer which are waiting to be transmitted. Bytes available in the
    *    TX FIFO do not count.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Number of bytes used in the TX buffer. Return value type depends on the TX 
    *  Buffer Size parameter.
    *
    * Global Variables:
    *  URT_txBufferWrite - used to calculate left space.
    *  URT_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 URT_GetTxBufferSize(void)
                                                            
    {
        uint8 size;

    #if (URT_TX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt. */
        URT_DisableTxInt();

        if(URT_txBufferRead == URT_txBufferWrite)
        {
            size = 0u;
        }
        else if(URT_txBufferRead < URT_txBufferWrite)
        {
            size = (URT_txBufferWrite - URT_txBufferRead);
        }
        else
        {
            size = (URT_TX_BUFFER_SIZE - URT_txBufferRead) +
                    URT_txBufferWrite;
        }

        URT_EnableTxInt();

    #else

        size = URT_TXSTATUS_REG;

        /* Is the fifo is full. */
        if((size & URT_TX_STS_FIFO_FULL) != 0u)
        {
            size = URT_FIFO_LENGTH;
        }
        else if((size & URT_TX_STS_FIFO_EMPTY) != 0u)
        {
            size = 0u;
        }
        else
        {
            /* We only know there is data in the fifo. */
            size = 1u;
        }

    #endif /* (URT_TX_INTERRUPT_ENABLED) */

    return(size);
    }


    /*******************************************************************************
    * Function Name: URT_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears all data from the TX buffer and hardware TX FIFO.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_txBufferWrite - cleared to zero.
    *  URT_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Data waiting in the transmit buffer is not sent; a byte that is currently
    *  transmitting finishes transmitting.
    *
    *******************************************************************************/
    void URT_ClearTxBuffer(void) 
    {
        uint8 enableInterrupts;

        enableInterrupts = CyEnterCriticalSection();
        /* Clear the HW FIFO */
        URT_TXDATA_AUX_CTL_REG |= (uint8)  URT_TX_FIFO_CLR;
        URT_TXDATA_AUX_CTL_REG &= (uint8) ~URT_TX_FIFO_CLR;
        CyExitCriticalSection(enableInterrupts);

    #if (URT_TX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt. */
        URT_DisableTxInt();

        URT_txBufferRead = 0u;
        URT_txBufferWrite = 0u;

        /* Enable Tx interrupt. */
        URT_EnableTxInt();

    #endif /* (URT_TX_INTERRUPT_ENABLED) */
    }


    /*******************************************************************************
    * Function Name: URT_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Transmits a break signal on the bus.
    *
    * Parameters:
    *  uint8 retMode:  Send Break return mode. See the following table for options.
    *   URT_SEND_BREAK - Initialize registers for break, send the Break
    *       signal and return immediately.
    *   URT_WAIT_FOR_COMPLETE_REINIT - Wait until break transmission is
    *       complete, reinitialize registers to normal transmission mode then return
    *   URT_REINIT - Reinitialize registers to normal transmission mode
    *       then return.
    *   URT_SEND_WAIT_REINIT - Performs both options: 
    *      URT_SEND_BREAK and URT_WAIT_FOR_COMPLETE_REINIT.
    *      This option is recommended for most cases.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  URT_initVar - checked to identify that the component has been
    *     initialized.
    *  txPeriod - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  There are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Function will block CPU until transmission
    *     complete.
    *  2) User may want to use blocking time if UART configured to the low speed
    *     operation
    *     Example for this case:
    *     SendBreak(0);     - initialize Break signal transmission
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to initialize and use the interrupt to
    *     complete break operation.
    *     Example for this case:
    *     Initialize TX interrupt with "TX - On TX Complete" parameter
    *     SendBreak(0);     - initialize Break signal transmission
    *         Add your code here to use CPU time
    *     When interrupt appear with URT_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *  The URT_SendBreak() function initializes registers to send a
    *  break signal.
    *  Break signal length depends on the break signal bits configuration.
    *  The register configuration should be reinitialized before normal 8-bit
    *  communication can continue.
    *
    *******************************************************************************/
    void URT_SendBreak(uint8 retMode) 
    {

        /* If not Initialized then skip this function*/
        if(URT_initVar != 0u)
        {
            /* Set the Counter to 13-bits and transmit a 00 byte */
            /* When that is done then reset the counter value back */
            uint8 tmpStat;

        #if(URT_HD_ENABLED) /* Half Duplex mode*/

            if( (retMode == URT_SEND_BREAK) ||
                (retMode == URT_SEND_WAIT_REINIT ) )
            {
                /* CTRL_HD_SEND_BREAK - sends break bits in HD mode */
                URT_WriteControlRegister(URT_ReadControlRegister() |
                                                      URT_CTRL_HD_SEND_BREAK);
                /* Send zeros */
                URT_TXDATA_REG = 0u;

                do /* Wait until transmit starts */
                {
                    tmpStat = URT_TXSTATUS_REG;
                }
                while((tmpStat & URT_TX_STS_FIFO_EMPTY) != 0u);
            }

            if( (retMode == URT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == URT_SEND_WAIT_REINIT) )
            {
                do /* Wait until transmit complete */
                {
                    tmpStat = URT_TXSTATUS_REG;
                }
                while(((uint8)~tmpStat & URT_TX_STS_COMPLETE) != 0u);
            }

            if( (retMode == URT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == URT_REINIT) ||
                (retMode == URT_SEND_WAIT_REINIT) )
            {
                URT_WriteControlRegister(URT_ReadControlRegister() &
                                              (uint8)~URT_CTRL_HD_SEND_BREAK);
            }

        #else /* URT_HD_ENABLED Full Duplex mode */

            static uint8 txPeriod;

            if( (retMode == URT_SEND_BREAK) ||
                (retMode == URT_SEND_WAIT_REINIT) )
            {
                /* CTRL_HD_SEND_BREAK - skip to send parity bit at Break signal in Full Duplex mode */
                #if( (URT_PARITY_TYPE != URT__B_UART__NONE_REVB) || \
                                    (URT_PARITY_TYPE_SW != 0u) )
                    URT_WriteControlRegister(URT_ReadControlRegister() |
                                                          URT_CTRL_HD_SEND_BREAK);
                #endif /* End URT_PARITY_TYPE != URT__B_UART__NONE_REVB  */

                #if(URT_TXCLKGEN_DP)
                    txPeriod = URT_TXBITCLKTX_COMPLETE_REG;
                    URT_TXBITCLKTX_COMPLETE_REG = URT_TXBITCTR_BREAKBITS;
                #else
                    txPeriod = URT_TXBITCTR_PERIOD_REG;
                    URT_TXBITCTR_PERIOD_REG = URT_TXBITCTR_BREAKBITS8X;
                #endif /* End URT_TXCLKGEN_DP */

                /* Send zeros */
                URT_TXDATA_REG = 0u;

                do /* Wait until transmit starts */
                {
                    tmpStat = URT_TXSTATUS_REG;
                }
                while((tmpStat & URT_TX_STS_FIFO_EMPTY) != 0u);
            }

            if( (retMode == URT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == URT_SEND_WAIT_REINIT) )
            {
                do /* Wait until transmit complete */
                {
                    tmpStat = URT_TXSTATUS_REG;
                }
                while(((uint8)~tmpStat & URT_TX_STS_COMPLETE) != 0u);
            }

            if( (retMode == URT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == URT_REINIT) ||
                (retMode == URT_SEND_WAIT_REINIT) )
            {

            #if(URT_TXCLKGEN_DP)
                URT_TXBITCLKTX_COMPLETE_REG = txPeriod;
            #else
                URT_TXBITCTR_PERIOD_REG = txPeriod;
            #endif /* End URT_TXCLKGEN_DP */

            #if( (URT_PARITY_TYPE != URT__B_UART__NONE_REVB) || \
                 (URT_PARITY_TYPE_SW != 0u) )
                URT_WriteControlRegister(URT_ReadControlRegister() &
                                                      (uint8) ~URT_CTRL_HD_SEND_BREAK);
            #endif /* End URT_PARITY_TYPE != NONE */
            }
        #endif    /* End URT_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: URT_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Configures the transmitter to signal the next bytes is address or data.
    *
    * Parameters:
    *  addressMode: 
    *       URT_SET_SPACE - Configure the transmitter to send the next
    *                                    byte as a data.
    *       URT_SET_MARK  - Configure the transmitter to send the next
    *                                    byte as an address.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  This function sets and clears URT_CTRL_MARK bit in the Control
    *  register.
    *
    *******************************************************************************/
    void URT_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable */
        if(addressMode != 0u)
        {
        #if( URT_CONTROL_REG_REMOVED == 0u )
            URT_WriteControlRegister(URT_ReadControlRegister() |
                                                  URT_CTRL_MARK);
        #endif /* End URT_CONTROL_REG_REMOVED == 0u */
        }
        else
        {
        #if( URT_CONTROL_REG_REMOVED == 0u )
            URT_WriteControlRegister(URT_ReadControlRegister() &
                                                  (uint8) ~URT_CTRL_MARK);
        #endif /* End URT_CONTROL_REG_REMOVED == 0u */
        }
    }

#endif  /* EndURT_TX_ENABLED */

#if(URT_HD_ENABLED)


    /*******************************************************************************
    * Function Name: URT_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Loads the receiver configuration in half duplex mode. After calling this
    *  function, the UART is ready to receive data.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  Valid only in half duplex mode. You must make sure that the previous
    *  transaction is complete and it is safe to unload the transmitter
    *  configuration.
    *
    *******************************************************************************/
    void URT_LoadRxConfig(void) 
    {
        URT_WriteControlRegister(URT_ReadControlRegister() &
                                                (uint8)~URT_CTRL_HD_SEND);
        URT_RXBITCTR_PERIOD_REG = URT_HD_RXBITCTR_INIT;

    #if (URT_RX_INTERRUPT_ENABLED)
        /* Enable RX interrupt after set RX configuration */
        URT_SetRxInterruptMode(URT_INIT_RX_INTERRUPTS_MASK);
    #endif /* (URT_RX_INTERRUPT_ENABLED) */
    }


    /*******************************************************************************
    * Function Name: URT_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Loads the transmitter configuration in half duplex mode. After calling this
    *  function, the UART is ready to transmit data.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  Valid only in half duplex mode. You must make sure that the previous
    *  transaction is complete and it is safe to unload the receiver configuration.
    *
    *******************************************************************************/
    void URT_LoadTxConfig(void) 
    {
    #if (URT_RX_INTERRUPT_ENABLED)
        /* Disable RX interrupts before set TX configuration */
        URT_SetRxInterruptMode(0u);
    #endif /* (URT_RX_INTERRUPT_ENABLED) */

        URT_WriteControlRegister(URT_ReadControlRegister() | URT_CTRL_HD_SEND);
        URT_RXBITCTR_PERIOD_REG = URT_HD_TXBITCTR_INIT;
    }

#endif  /* URT_HD_ENABLED */


/* [] END OF FILE */
